"use strict";
/**
 * @class elFinder command "info" updated for Exponent CMS
 * Display dialog with file properties.
 *
 * @author Dmitry (dio) Levashov, dio@std42.ru
 **/
elFinder.prototype.commands.info = function () {
    var m = 'msg',
        fm = this.fm,
        spclass = 'elfinder-info-spinner',
		btnclass = 'elfinder-info-button',
        msg = {
            calc : fm.i18n('calc') ,
            size : fm.i18n('size') ,
            unknown : fm.i18n('unknown') ,
            path : fm.i18n('path') ,
            aliasfor : fm.i18n('aliasfor') ,
            modify : fm.i18n('modify') ,
            perms : fm.i18n('perms') ,
            locked : fm.i18n('locked') ,
            dim : fm.i18n('dim') ,
            kind : fm.i18n('kind') ,
            files : fm.i18n('files') ,
            folders : fm.i18n('folders') ,
            items : fm.i18n('items') ,
            yes : fm.i18n('yes') ,
            no : fm.i18n('no') ,
            link : fm.i18n('link') ,
            owner : fm.i18n('owner') ,
            group    : fm.i18n('group'),
            perm     : fm.i18n('perm'),
            getlink  : fm.i18n('getLink'),
            shared : fm.i18n('shared') ,
            title : fm.i18n('title') ,
            alt : fm.i18n('alt')
        };

    this.tpl = {
        main : '<div class="ui-helper-clearfix elfinder-info-title"><span class="elfinder-cwd-icon {class} ui-corner-all"/>{title}</div><table class="elfinder-info-tb">{content}</table>' ,
        itemTitle : '<strong>{name}</strong><span class="elfinder-info-kind">{kind}</span>' ,
        groupTitle : '<strong>{items}: {num}</strong>' ,
        row : '<tr><td>{label} : </td><td>{value}</td></tr>' ,
		spinner    : '<span>{text}</span> <span class="'+spclass+' '+spclass+'-{name}"/>'
    };

    this.alwaysEnabled = true;
    this.updateOnSelect = false;
    this.shortcuts = [
        {
            pattern : 'ctrl+i'
        }
    ];

    this.init = function () {
        $.each(msg , function (k , v) {
            msg[k] = fm.i18n(v);
        });
    };

    this.getstate = function () {
        return 0;
    };

    this.exec = function (hashes) {
        var files = this.files(hashes);
        if (!files.length) {
            files = this.files([ this.fm.cwd().hash ]);
        }
        var self = this,
            fm = this.fm,
			o       = this.options,
            tpl = this.tpl,
            row = tpl.row,
            cnt = files.length,
            content = [],
            view = tpl.main,
            l = '{label}',
            v = '{value}',
			reqs    = [],
            opts = {
                title : this.title ,
                width : 'auto' ,
                modal : true ,  //exp
				close : function() {
					$(this).elfinderdialog('destroy');
					$.each(reqs, function(i, req) {
						var xhr = (req && req.xhr)? req.xhr : null;
						if (xhr && xhr.state() == 'pending') {
							xhr.quiet = true;
							xhr.abort();
						}
					});
				}
            },
            count = [],
			replSpinner = function(msg, name) { dialog.find('.'+spclass+'-'+name).parent().html(msg); },
			id = fm.namespace+'-info-'+$.map(files, function(f) { return f.hash; }).join('-'),
            dialog = fm.getUI().find('#' + id),
			customActions = [],
            size, tmb, file, title, dcnt;

        if (!cnt) {
            return $.Deferred().reject();
        }

        if (dialog.length) {
            dialog.elfinderdialog('toTop');
            return $.Deferred().resolve();
        }


        if (cnt == 1) {
            file = files[0];

            view = view.replace('{class}' , fm.mime2class(file.mime));
			title = tpl.itemTitle.replace('{name}', fm.escape(file.i18 || file.name)).replace('{kind}', '<span title="'+fm.escape(file.mime)+'">'+fm.mime2kind(file)+'</span>');

			tmb = fm.tmb(file);

            if (!file.read) {
                size = msg.unknown;
            } else if (file.mime != 'directory' || file.alias) {
                size = fm.formatSize(file.size);
            } else {
				size = tpl.spinner.replace('{text}', msg.calc).replace('{name}', 'size');
                count.push(file.hash);
            }

            content.push(row.replace(l , msg.size).replace(v , size));
            file.alias && content.push(row.replace(l , msg.aliasfor).replace(v , file.alias));
            content.push(row.replace(l , msg.path).replace(v , fm.escape(fm.path(file.hash , true))));
			if (file.read) {
				var href,
				name_esc = fm.escape(file.name);
				if (file.url == '1') {
                    content.push(row.replace(l, msg.link).replace(v, '<button class="'+btnclass+' '+spclass+'-url">'+msg.getlink+'</button>'));
				} else {
                    if (o.nullUrlDirLinkSelf && file.mime == 'directory' && file.url === null) {
                        var loc = window.location;
                        href = loc.pathname + loc.search + '#elf_' + file.hash;
                    } else {
                        href = fm.url(file.hash);
                    }
					content.push(row.replace(l, msg.link).replace(v,  '<a href="'+href+'" target="_blank">'+name_esc+'</a>'));
                }
			}

            if (file.dim) { // old api
                content.push(row.replace(l , msg.dim).replace(v , file.dim));
            } else if (file.mime.indexOf('image') !== -1) {
                if (file.width && file.height) {
                    content.push(row.replace(l , msg.dim).replace(v , file.width + 'x' + file.height));
                } else {
					content.push(row.replace(l, msg.dim).replace(v, tpl.spinner.replace('{text}', msg.calc).replace('{name}', 'dim')));
					reqs.push(fm.request({
                        data : {cmd : 'dim' , target : file.hash} ,
                        preventDefault : true
                    })
                        .fail(function () {
						replSpinner(msg.unknown, 'dim');
                        })
                        .done(function (data) {
						replSpinner(data.dim || msg.unknown, 'dim');
                            if (data.dim) {
                                var dim = data.dim.split('x');
                                var rfile = fm.file(file.hash);
                                rfile.width = dim[0];
                                rfile.height = dim[1];
                            }
					}));
                }
            }


            content.push(row.replace(l , msg.modify).replace(v , fm.formatDate(file)));
            content.push(row.replace(l , msg.perms).replace(v , fm.formatPermissions(file)));
            content.push(row.replace(l , msg.locked).replace(v , file.locked ? msg.yes : msg.no));
			//file.owner && content.push(row.replace(l, msg.owner).replace(v, file.owner));
            file.group && content.push(row.replace(l, msg.group).replace(v, file.group));
            file.perm && content.push(row.replace(l, msg.perm).replace(v, fm.formatFileMode(file.perm)));

			// Add custom info fields
			if (o.custom) {
				$.each(o.custom, function(name, details) {
					if (
					  (!details.mimes || $.map(details.mimes, function(m){return (file.mime === m || file.mime.indexOf(m+'/') === 0)? true : null;}).length)
					    &&
					  (!details.hashRegex || file.hash.match(details.hashRegex))
					) {
						// Add to the content
						content.push(row.replace(l, fm.i18n(details.label)).replace(v , details.tpl.replace('{id}', id)));
						// Register the action
						if (details.action && (typeof details.action == 'function')) {
							customActions.push(details.action);
						}
					}
				});
			}

            // Exponent specific attributes
            var editDesc = true;
            if (file.mime != 'directory') {
                content.push(row.replace(l , 'Owner').replace(v , file.owner));

                var shared = '';
                if (file.shared) shared = ' checked="true"';
                var disableshared = '';
                if (!editDesc) disableshared = ' disabled="true"';
                var fileshared = '<input type="checkbox" id="elfinder-fm-file-shared" class="ui-widget ui-widget-content" title="Change the file\'s Shared status" value="1"' + shared + disableshared + '" />';
                content.push(row.replace(l , 'Shared').replace(v , fileshared));
            }

            if (file.mime.indexOf('image') !== -1) {
                var filetitle = '<div id="elfinder-fm-file-title">' + file.title + '</div>';
                var filealt = '<div id="elfinder-fm-file-alt">' + file.alt + '</div>';
                if (editDesc) {
                    filetitle = '<input size="23" id="elfinder-fm-file-title" class="ui-widget ui-widget-content" value="' + file.title + '" /> <input type="button" id="elfinder-fm-file-title-btn-save" title="Update the file\'s Title" value="' + 'Update' + '" />';
                    filealt = '<textarea cols="20" rows="2" id="elfinder-fm-file-alt" class="ui-widget ui-widget-content">' + file.alt + '</textarea> <input type="button" id="elfinder-fm-file-alt-btn-save" title="Update the file\'s Alt" value="' + 'Update' + '" />';
                }
                content.push(row.replace(l , 'Title').replace(v , filetitle));
                content.push(row.replace(l , 'Alt').replace(v , filealt));
            }

        } else {
            view = view.replace('{class}' , 'elfinder-cwd-icon-group');
            title = tpl.groupTitle.replace('{items}' , msg.items).replace('{num}' , cnt);
            dcnt = $.map(files ,function (f) {
                return f.mime == 'directory' ? 1 : null;
            }).length;
            if (!dcnt) {
                size = 0;
                $.each(files , function (h , f) {
                    var s = parseInt(f.size);

                    if (s >= 0 && size >= 0) {
                        size += s;
                    } else {
                        size = 'unknown';
                    }
                });
                content.push(row.replace(l , msg.kind).replace(v , msg.files));
                content.push(row.replace(l , msg.size).replace(v , fm.formatSize(size)));
            } else {
                content.push(row.replace(l , msg.kind).replace(v , dcnt == cnt ? msg.folders : msg.folders + ' ' + dcnt + ', ' + msg.files + ' ' + (cnt - dcnt)));
				content.push(row.replace(l, msg.size).replace(v, tpl.spinner.replace('{text}', msg.calc).replace('{name}', 'size')));
				count = $.map(files, function(f) { return f.hash; });

            }
        }

        view = view.replace('{title}' , title).replace('{content}' , content.join(''));

        dialog = fm.dialog(view , opts);
        dialog.attr('id' , id);

		if (file && file.url == '1') {
            dialog.on('click', '.'+spclass+'-url', function(){
                $(this).parent().html(tpl.spinner.replace('{text}', fm.i18n('ntfurl')).replace('{name}', 'url'));
                fm.request({
                    data : {cmd : 'url', target : file.hash},
                    preventDefault : true
                })
                .fail(function() {
                    replSpinner(name_esc, 'url');
                })
                .done(function(data) {
                    if (data.url) {
                        replSpinner('<a href="'+data.url+'" target="_blank">'+name_esc+'</a>' || name_esc, 'url');
                        var rfile = fm.file(file.hash);
                        rfile.url = data.url;
                    } else {
                        replSpinner(name_esc, 'url');
                    }
                });
            });
        }

        // Exponent specific commands
        if (editDesc) {
            var checkShared = $('#elfinder-fm-file-shared' , dialog);
            checkShared.click(function () {
                fm.lockfiles({files : [file.hash]});
                fm.request({
                    data : {
                        cmd : 'shared' ,
                        target : file.hash ,
                        content : checkShared[0].checked
                    } ,
//                    notify : {
//                        type : 'save' ,
//                        cnt : 1
//                    }
                })
                    .always(function () {
                        fm.unlockfiles({files : [file.hash]});
                        file.shared = checkShared[0].checked;
                    });
            });

            if (file.mime.indexOf('image') !== -1) {
                var inputTitle = $('#elfinder-fm-file-title' , dialog);
                var btnTitleSave = $('#elfinder-fm-file-title-btn-save' , dialog).button();
                var inputAlt = $('#elfinder-fm-file-alt' , dialog);
                var btnAltSave = $('#elfinder-fm-file-alt-btn-save' , dialog).button();

                btnTitleSave.click(function () {
                    fm.lockfiles({files : [file.hash]});
                    fm.request({
                        data : {
                            cmd : 'title' ,
                            target : file.hash ,
                            content : inputTitle.val()
                        } ,
                        //                    notify : {
                        //                        type : 'save' ,
                        //                        cnt : 1
                        //                    }
                    })
                        .always(function () {
                            fm.unlockfiles({files : [file.hash]})
                            file.title = inputTitle.val();
                        });
                });
                btnAltSave.click(function () {
                    fm.lockfiles({files : [file.hash]});
                    fm.request({
                        data : {
                            cmd : 'alt' ,
                            target : file.hash ,
                            content : inputAlt.val()
                        } ,
                        //                    notify : {
                        //                        type : 'save' ,
                        //                        cnt : 1
                        //                    }
                    })
                        .always(function () {
                            fm.unlockfiles({files : [file.hash]})
                            file.alt = inputAlt.val();
                        });
                });
            }
        }

        // load thumbnail
        if (tmb) {
            $('<img/>')
				.load(function() { dialog.find('.elfinder-cwd-icon').addClass(tmb.className).css('background-image', "url('"+tmb.url+"')"); })
				.attr('src', tmb.url);
        }

        // send request to count total size
        if (count.length) {
			reqs.push(fm.request({
                data : {cmd : 'size' , targets : count} ,
                preventDefault : true
            })
                .fail(function () {
					replSpinner(msg.unknown, 'size');
                })
                .done(function (data) {
                    var size = parseInt(data.size);
					replSpinner(size >= 0 ? fm.formatSize(size) : msg.unknown, 'size');
				})
			);
        }

		// call custom actions
		if (customActions.length) {
			$.each(customActions, function(i, action) {
				try {
					action(file, fm, dialog);
				} catch(e) {
					fm.debug('error', e);
				}
			});
		}

    };

};
